@@ -35,11 +35,11 @@ Torus Knot Software Ltd.
 #include "OgreSingleton.h"
 #include "OgreHardwareVertexBuffer.h"
 #include "OgreHardwareIndexBuffer.h"
-
+#include "OgreRenderToVertexBuffer.h"
 
 namespace Ogre {
 
-    /** Abtract interface representing a 'licensee' of a hardware buffer copy.
+    /** Abstract interface representing a 'licensee' of a hardware buffer copy.
     remarks
         Often it's useful to have temporary buffers which are used for working
         but are not necessarily needed permanently. However, creating and 
@@ -59,7 +59,7 @@ namespace Ogre {
     };
 
     /** Structure for recording the use of temporary blend buffers */
-    class _OgreExport TempBlendedBufferInfo : public HardwareBufferLicensee
+    class _OgreExport TempBlendedBufferInfo : public HardwareBufferLicensee, public BufferAlloc
     {
     private:
         // Pre-blended 
@@ -228,7 +228,7 @@ namespace Ogre {
 			bool useShadowBuffer = false) = 0;
 		/** Create a hardware index buffer.
         @remarks Note that because buffers can be shared, they are reference
-            counted so you do not need to worry about destroying themm this will be done
+            counted so you do not need to worry about destroying them this will be done
             automatically.
 		@param itype The type in index, either 16- or 32-bit, depending on how many vertices
 			you need to be able to address
@@ -246,6 +246,12 @@ namespace Ogre {
             createIndexBuffer(HardwareIndexBuffer::IndexType itype, size_t numIndexes, 
 			HardwareBuffer::Usage usage, bool useShadowBuffer = false) = 0;
 
+		/** Create a render to vertex buffer.
+		@remarks The parameters (such as vertex size etc) are determined later
+			and are allocated when needed.
+		*/
+		virtual RenderToVertexBufferSharedPtr createRenderToVertexBuffer() = 0;
+
         /** Creates a new vertex declaration. */
         virtual VertexDeclaration* createVertexDeclaration(void);
         /** Destroys a vertex declaration. */
