@@ -38,11 +38,11 @@ Torus Knot Software Ltd.
 #include "OgrePlane.h"
 namespace Ogre
 {
-    /** Class encapsulating a standard 4x4 homogenous matrix.
+    /** Class encapsulating a standard 4x4 homogeneous matrix.
         @remarks
             OGRE uses column vectors when applying matrix multiplications,
             This means a vector is represented as a single column, 4-row
-            matrix. This has the effect that the tranformations implemented
+            matrix. This has the effect that the transformations implemented
             by the matrices happens right-to-left e.g. if vector V is to be
             transformed by M1 then M2 then M3, the calculation would be
             M3 * M2 * M1 * V. The order that matrices are concatenated is
@@ -447,7 +447,30 @@ namespace Ogre
 
         }
 
-        /** Extracts the rotation / scaling part as a quaternion from the Matrix.
+		/** Determines if this matrix involves a scaling. */
+		inline bool hasScale() const
+		{
+			// check magnitude of column vectors (==local axes)
+			Real t = m[0][0] * m[0][0] + m[1][0] * m[1][0] + m[2][0] * m[2][0];
+			if (!Math::RealEqual(t, 1.0, 1e-04))
+				return true;
+			t = m[0][1] * m[0][1] + m[1][1] * m[1][1] + m[2][1] * m[2][1];
+			if (!Math::RealEqual(t, 1.0, 1e-04))
+				return true;
+			t = m[0][2] * m[0][2] + m[1][2] * m[1][2] + m[2][2] * m[2][2];
+			if (!Math::RealEqual(t, 1.0, 1e-04))
+				return true;
+
+			return false;
+		}
+
+		/** Determines if this matrix involves a negative scaling. */
+		inline bool hasNegativeScale() const
+		{
+			return determinant() < 0;
+		}
+
+		/** Extracts the rotation / scaling part as a quaternion from the Matrix.
          */
         inline Quaternion extractQuaternion() const
         {
