@@ -39,6 +39,8 @@ Torus Knot Software Ltd.
 #include "OgreCompositorManager.h"
 #include "OgreSceneManager.h"
 #include "OgreRenderQueueInvocation.h"
+#include "OgreMaterialManager.h"
+
 namespace Ogre {
 CompositorChain::CompositorChain(Viewport *vp):
     mViewport(vp),
@@ -62,10 +64,10 @@ void CompositorChain::destroyResources(void)
 	if (mViewport)
 	{
 		removeAllCompositors();
-		mViewport->getTarget()->removeListener(this);
 		/// Destroy "original scene" compositor instance
 		if (mOriginalScene)
 		{
+			mViewport->getTarget()->removeListener(this);
 			mOriginalScene->getTechnique()->destroyInstance(mOriginalScene);
 			mOriginalScene = 0;
 		}
@@ -186,6 +188,11 @@ void CompositorChain::preRenderTargetUpdate(const RenderTargetEvent& evt)
 	/// ( RenderSystem::setViewport(...) ) if it would have been, the rendering
 	/// order would be screwed up and problems would arise with copying rendertextures.
     Camera *cam = mViewport->getCamera();
+	if (!cam)
+	{
+		return;
+	}
+
     /// Iterate over compiled state
     CompositorInstance::CompiledState::iterator i;
     for(i=mCompiledState.begin(); i!=mCompiledState.end(); ++i)
@@ -226,8 +233,11 @@ void CompositorChain::preViewportUpdate(const RenderTargetViewportEvent& evt)
 	}
 
 	Camera *cam = mViewport->getCamera();
-	/// Prepare for output operation
-	preTargetOperation(mOutputOperation, mViewport, cam);
+	if (cam)
+	{
+		/// Prepare for output operation
+		preTargetOperation(mOutputOperation, mViewport, cam);
+	}
 }
 //-----------------------------------------------------------------------
 void CompositorChain::preTargetOperation(CompositorInstance::TargetOperation &op, Viewport *vp, Camera *cam)
@@ -264,8 +274,6 @@ void CompositorChain::postTargetOperation(CompositorInstance::TargetOperation &o
     SceneManager *sm = cam->getSceneManager();
 	/// Unregister our listener
 	sm->removeRenderQueueListener(&mOurListener);
-	/// Flush remaing operations
-	mOurListener.flushUpTo((uint8)RENDER_QUEUE_COUNT);
 	/// Restore default scene and camera settings
 	sm->setVisibilityMask(mOldVisibilityMask);
 	sm->setFindVisibleObjects(mOldFindVisibleObjects);
@@ -280,7 +288,11 @@ void CompositorChain::postViewportUpdate(const RenderTargetViewportEvent& evt)
     if(evt.source != mViewport || !mAnyCompositorsEnabled)
         return;
 
-	postTargetOperation(mOutputOperation, mViewport, mViewport->getCamera());
+	Camera *cam = mViewport->getCamera();
+	if (cam)
+	{
+		postTargetOperation(mOutputOperation, mViewport, cam);
+	}
 }
 //-----------------------------------------------------------------------
 void CompositorChain::viewportRemoved(const RenderTargetViewportEvent& evt)
@@ -316,6 +328,11 @@ void CompositorChain::_compile()
 
 	bool compositorsEnabled = false;
 
+	// force default scheme so materials for compositor quads will determined correctly
+	MaterialManager& matMgr = MaterialManager::getSingleton();
+	String prevMaterialScheme = matMgr.getActiveScheme();
+	matMgr.setActiveScheme(MaterialManager::DEFAULT_SCHEME_NAME);
+	
     /// Set previous CompositorInstance for each compositor in the list
     CompositorInstance *lastComposition = mOriginalScene;
 	mOriginalScene->mPreviousInstance = 0;
@@ -358,6 +375,10 @@ void CompositorChain::_compile()
 				mOldClearEveryFrameBuffers);
 		}
 	}
+
+	// restore material scheme
+	matMgr.setActiveScheme(prevMaterialScheme);
+
     
     mDirty = false;
 }
