@@ -1174,8 +1174,7 @@ namespace Ogre {
 			qmesh->worldBounds.getMinimum() ,
 			qmesh->worldBounds.getMaximum());
 		mAABB.merge(localBounds);
-		mBoundingRadius = std::max(mBoundingRadius, localBounds.getMinimum().length());
-		mBoundingRadius = std::max(mBoundingRadius, localBounds.getMaximum().length());
+		mBoundingRadius = Math::boundingRadiusFromAABB(mAABB);
 
 	}
 	//--------------------------------------------------------------------------
@@ -1587,6 +1586,7 @@ namespace Ogre {
 	//--------------------------------------------------------------------------
 	void InstancedGeometry::MaterialBucket::build()
 	{
+		mTechnique = 0;
 		mMaterial = MaterialManager::getSingleton().getByName(mMaterialName);
 		if (mMaterial.isNull())
 		{
