@@ -42,6 +42,7 @@ Torus Knot Software Ltd.
 #include "OgreAnimationState.h"
 #include "OgreAnimationTrack.h"
 #include "OgreOptimisedUtil.h"
+#include "OgreTangentSpaceCalc.h"
 
 namespace Ogre {
     //-----------------------------------------------------------------------
@@ -132,11 +133,14 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     SubMesh* Mesh::createSubMesh()
     {
-        SubMesh* sub = new SubMesh();
+        SubMesh* sub = OGRE_NEW SubMesh();
         sub->parent = this;
 
         mSubMeshList.push_back(sub);
 
+		if (isLoaded())
+			_dirtyState();
+
         return sub;
     }
     //-----------------------------------------------------------------------
@@ -194,16 +198,41 @@ namespace Ogre {
 		}
 	}
 	//-----------------------------------------------------------------------
-    void Mesh::loadImpl()
+    void Mesh::prepareImpl()
     {
         // Load from specified 'name'
-        MeshSerializer serializer;
-        LogManager::getSingleton().logMessage("Mesh: Loading " + mName + ".");
+        if (getCreator()->getVerbose())
+            LogManager::getSingleton().logMessage("Mesh: Loading "+mName+".");
 
-        DataStreamPtr stream =
+        mFreshFromDisk =
             ResourceGroupManager::getSingleton().openResource(
 				mName, mGroup, true, this);
-        serializer.importMesh(stream, this);
+ 
+        // fully prebuffer into host RAM
+        mFreshFromDisk = DataStreamPtr(OGRE_NEW MemoryDataStream(mName,mFreshFromDisk));
+    }
+    //-----------------------------------------------------------------------
+    void Mesh::unprepareImpl()
+    {
+        mFreshFromDisk.setNull();
+    }
+    void Mesh::loadImpl()
+    {
+        MeshSerializer serializer;
+        serializer.setListener(MeshManager::getSingleton().getListener());
+
+        // If the only copy is local on the stack, it will be cleaned
+        // up reliably in case of exceptions, etc
+        DataStreamPtr data(mFreshFromDisk);
+        mFreshFromDisk.setNull();
+
+        if (data.isNull()) {
+            OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
+                        "Data doesn't appear to have been prepared in " + mName,
+                        "Mesh::loadImpl()");
+        }
+
+		serializer.importMesh(data, this);
 
         /* check all submeshes to see if their materials should be
            updated.  If the submesh has texture aliases that match those
@@ -220,11 +249,11 @@ namespace Ogre {
         for (SubMeshList::iterator i = mSubMeshList.begin();
             i != mSubMeshList.end(); ++i)
         {
-            delete *i;
+            OGRE_DELETE *i;
         }
         if (sharedVertexData)
         {
-            delete sharedVertexData;
+            OGRE_DELETE sharedVertexData;
             sharedVertexData = NULL;
         }
 		// Clear SubMesh lists
@@ -285,7 +314,7 @@ namespace Ogre {
             }
 
             // Copy index data
-            delete newSub->indexData;
+            OGRE_DELETE newSub->indexData;
 			newSub->indexData = (*subi)->indexData->clone();
             // Copy any bone assignments
             newSub->mBoneAssignments = (*subi)->mBoneAssignments;
@@ -349,7 +378,7 @@ namespace Ogre {
 			i != mAnimationsList.end(); ++i)
 		{
 			Animation *newAnim = i->second->clone(i->second->getName());
-			newMesh->mAnimationsList[newName] = newAnim;
+			newMesh->mAnimationsList[i->second->getName()] = newAnim;
 		}
 		// Clone pose list
 		for (PoseList::iterator i = mPoseList.begin(); i != mPoseList.end(); ++i)
@@ -358,7 +387,7 @@ namespace Ogre {
 			newMesh->mPoseList.push_back(newPose);
 		}
 		newMesh->mSharedVertexDataAnimationType = mSharedVertexDataAnimationType;
-		newMesh->mAnimationTypesDirty = mAnimationTypesDirty;
+		newMesh->mAnimationTypesDirty = true;
 
 
         newMesh->load();
@@ -378,13 +407,8 @@ namespace Ogre {
         mAABB = bounds;
         Vector3 max = mAABB.getMaximum();
         Vector3 min = mAABB.getMinimum();
+		mBoundRadius = Math::boundingRadiusFromAABB(mAABB);
 
-        // Set sphere bounds; not the tightest by since we're using
-        // manual AABB it is the only way
-        Real sqLen1 = min.squaredLength();
-        Real sqLen2 = max.squaredLength();
-
-        mBoundRadius = Math::Sqrt(std::max(sqLen1, sqLen2));
         if (pad)
         {
             // Pad out the AABB a little, helps with most bounds tests
@@ -408,33 +432,38 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Mesh::setSkeletonName(const String& skelName)
     {
-        mSkeletonName = skelName;
+		if (skelName != mSkeletonName)
+		{
+			mSkeletonName = skelName;
 
-        if (skelName.empty())
-        {
-            // No skeleton
-            mSkeleton.setNull();
-        }
-        else
-        {
-            // Load skeleton
-            try {
-                mSkeleton = SkeletonManager::getSingleton().load(skelName, mGroup);
-            }
-            catch (...)
-            {
-                mSkeleton.setNull();
-                // Log this error
-                String msg = "Unable to load skeleton ";
-                msg += skelName + " for Mesh " + mName
-                    + ". This Mesh will not be animated. "
-                    + "You can ignore this message if you are using an offline tool.";
-                LogManager::getSingleton().logMessage(msg);
+			if (skelName.empty())
+			{
+				// No skeleton
+				mSkeleton.setNull();
+			}
+			else
+			{
+				// Load skeleton
+				try {
+					mSkeleton = SkeletonManager::getSingleton().load(skelName, mGroup);
+				}
+				catch (...)
+				{
+					mSkeleton.setNull();
+					// Log this error
+					String msg = "Unable to load skeleton ";
+					msg += skelName + " for Mesh " + mName
+						+ ". This Mesh will not be animated. "
+						+ "You can ignore this message if you are using an offline tool.";
+					LogManager::getSingleton().logMessage(msg);
 
-            }
+				}
 
 
-        }
+			}
+			if (isLoaded())
+				_dirtyState();
+		}
     }
     //-----------------------------------------------------------------------
     bool Mesh::hasSkeleton(void) const
@@ -836,24 +865,35 @@ namespace Ogre {
 
         removeLodLevels();
 
-		StringUtil::StrStreamType str;
-		str << "Generating " << lodDistances.size()
+		LogManager::getSingleton().stream()
+			<< "Generating " << lodDistances.size()
 			<< " lower LODs for mesh " << mName;
-        LogManager::getSingleton().logMessage(str.str());
 
         SubMeshList::iterator isub, isubend;
         isubend = mSubMeshList.end();
         for (isub = mSubMeshList.begin(); isub != isubend; ++isub)
         {
-            // Set up data for reduction
-            VertexData* pVertexData = (*isub)->useSharedVertices ? sharedVertexData : (*isub)->vertexData;
+            // check if triangles are present
+            if ((*isub)->indexData->indexCount > 0)
+            {
+                // Set up data for reduction
+                VertexData* pVertexData = (*isub)->useSharedVertices ? sharedVertexData : (*isub)->vertexData;
 
-            ProgressiveMesh pm(pVertexData, (*isub)->indexData);
-            pm.build(
-            static_cast<ushort>(lodDistances.size()),
-                &((*isub)->mLodFaceList),
-                reductionMethod, reductionValue);
+                ProgressiveMesh pm(pVertexData, (*isub)->indexData);
+                pm.build(
+                static_cast<ushort>(lodDistances.size()),
+                    &((*isub)->mLodFaceList),
+                    reductionMethod, reductionValue);
 
+            }
+            else
+            {
+                // create empty index data for each lod
+                for (size_t i = 0; i < lodDistances.size(); ++i)
+                {
+                    (*isub)->mLodFaceList.push_back(OGRE_NEW IndexData);
+                }
+            }
         }
 
         // Iterate over the lods and record usage
@@ -897,12 +937,11 @@ namespace Ogre {
 			}
 			catch (Exception& )
 			{
-				StringUtil::StrStreamType str;
-				str << "Error while loading manual LOD level "
+				LogManager::getSingleton().stream()
+					<< "Error while loading manual LOD level "
 					<< mMeshLodUsageList[index].manualName
 					<< " - this LOD level will not be rendered. You can "
 					<< "ignore this error in offline mesh tools.";
-				LogManager::getSingleton().logMessage(str.str());
 			}
 
         }
@@ -949,7 +988,7 @@ namespace Ogre {
 
 		lod->manualName = meshName;
 		lod->manualMesh.setNull();
-        if (lod->edgeData) delete lod->edgeData;
+        if (lod->edgeData) OGRE_DELETE lod->edgeData;
         lod->edgeData = 0;
 	}
     //---------------------------------------------------------------------
@@ -1159,251 +1198,97 @@ namespace Ogre {
     }
     //---------------------------------------------------------------------
     void Mesh::buildTangentVectors(VertexElementSemantic targetSemantic, 
-		unsigned short sourceTexCoordSet, unsigned short index)
+		unsigned short sourceTexCoordSet, unsigned short index, 
+		bool splitMirrored, bool splitRotated, bool storeParityInW)
     {
-		if (index == 0 && targetSemantic == VES_TEXTURE_COORDINATES)
+
+		TangentSpaceCalc tangentsCalc;
+		tangentsCalc.setSplitMirrored(splitMirrored);
+		tangentsCalc.setSplitRotated(splitRotated);
+		tangentsCalc.setStoreParityInW(storeParityInW);
+
+		// shared geometry first
+		if (sharedVertexData)
 		{
-			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
-				"Destination texture coordinate set must be greater than 0",
-				"Mesh::buildTangentVectors");
+			tangentsCalc.setVertexData(sharedVertexData);
+			bool found = false;
+			for (SubMeshList::iterator i = mSubMeshList.begin(); i != mSubMeshList.end(); ++i)
+			{
+				SubMesh* sm = *i;
+				if (sm->useSharedVertices)
+				{
+					tangentsCalc.addIndexData(sm->indexData);
+					found = true;
+				}
+			}
+			if (found)
+			{
+				TangentSpaceCalc::Result res = 
+					tangentsCalc.build(targetSemantic, sourceTexCoordSet, index);
+
+				// If any vertex splitting happened, we have to give them bone assignments
+				if (getSkeletonName() != StringUtil::BLANK)
+				{
+					for (TangentSpaceCalc::IndexRemapList::iterator r = res.indexesRemapped.begin(); 
+						r != res.indexesRemapped.end(); ++r)
+					{
+						TangentSpaceCalc::IndexRemap& remap = *r;
+						// Copy all bone assignments from the split vertex
+						VertexBoneAssignmentList::iterator vbstart = mBoneAssignments.lower_bound(remap.splitVertex.first);
+						VertexBoneAssignmentList::iterator vbend = mBoneAssignments.upper_bound(remap.splitVertex.first);
+						for (VertexBoneAssignmentList::iterator vba = vbstart; vba != vbend; ++vba)
+						{
+							VertexBoneAssignment newAsgn = vba->second;
+							newAsgn.vertexIndex = static_cast<unsigned int>(remap.splitVertex.second);
+							// multimap insert doesn't invalidate iterators
+							addBoneAssignment(newAsgn);
+						}
+						
+					}
+
+				}
+			}
 		}
 
-	    // our temp. buffers
-	    uint32			vertInd[3];
-	    Vector3         vertPos[3];
-        Real            u[3], v[3];
-	    // setup a new 3D texture coord-set buffer for every sub mesh
-	    int nSubMesh = getNumSubMeshes();
-	    for (int sm = 0; sm < nSubMesh; sm++)
-	    {
-		    // retrieve buffer pointers
-		    uint16	*pVIndices16 = NULL;    // the face indices buffer, read only
-		    uint32	*pVIndices32 = NULL;    // the face indices buffer, read only
-		    float	*p2DTC;	                // pointer to 2D tex.coords, read only
-		    float	*p3DTC;	                // pointer to 3D tex.coords, write/read (discard)
-		    float	*pVPos;	                // vertex position buffer, read only
-			float	*pVNorm;	            // vertex normal buffer, read only
-
-		    SubMesh *pSubMesh = getSubMesh(sm);
-
-		    // retrieve buffer pointers
-		    // first, indices
-		    IndexData *indexData = pSubMesh->indexData;
-		    HardwareIndexBufferSharedPtr buffIndex = indexData->indexBuffer;
-			bool use32bit = false;
-			if (buffIndex->getType() == HardwareIndexBuffer::IT_32BIT)
+		// Dedicated geometry
+		for (SubMeshList::iterator i = mSubMeshList.begin(); i != mSubMeshList.end(); ++i)
+		{
+			SubMesh* sm = *i;
+			if (!sm->useSharedVertices)
 			{
-		    	pVIndices32 = static_cast<uint32*>(
-					buffIndex->lock(HardwareBuffer::HBL_READ_ONLY));
-				use32bit = true;
-			}
-			else
-			{
-		    	pVIndices16 = static_cast<uint16*>(
-					buffIndex->lock(HardwareBuffer::HBL_READ_ONLY));
-			}
-		    // then, vertices
-		    VertexData *usedVertexData ;
-		    if (pSubMesh->useSharedVertices) {
-			    usedVertexData = sharedVertexData;
-		    } else {
-			    usedVertexData = pSubMesh->vertexData;
-		    }
-		    VertexDeclaration *vDecl = usedVertexData->vertexDeclaration;
-		    VertexBufferBinding *vBind = usedVertexData->vertexBufferBinding;
+				tangentsCalc.clear();
+				tangentsCalc.setVertexData(sm->vertexData);
+				tangentsCalc.addIndexData(sm->indexData);
+				TangentSpaceCalc::Result res = 
+					tangentsCalc.build(targetSemantic, sourceTexCoordSet, index);
 
-
-		    // make sure we have a 3D coord to place data in
-			organiseTangentsBuffer(usedVertexData, targetSemantic, index, sourceTexCoordSet);
-
-            // Get the target element
-            const VertexElement* destElem = vDecl->findElementBySemantic(
-				targetSemantic, index);
-            // Get the source element
-            const VertexElement* srcElem = vDecl->findElementBySemantic(
-				VES_TEXTURE_COORDINATES, sourceTexCoordSet);
-
-            if (!srcElem || srcElem->getType() != VET_FLOAT2)
-            {
-                OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
-                    "SubMesh " + StringConverter::toString(sm) + " of Mesh " + mName +
-                    " has no 2D texture coordinates at the selected set, therefore we cannot calculate tangents.",
-                    "Mesh::buildTangentVectors");
-            }
-            HardwareVertexBufferSharedPtr srcBuf, destBuf, posBuf, normBuf;
-            unsigned char *pSrcBase, *pDestBase, *pPosBase, *pNormBase;
-            size_t srcInc, destInc, posInc, normInc;
-
-            srcBuf = vBind->getBuffer(srcElem->getSource());
-            // Is the source and destination buffer the same?
-            if (srcElem->getSource() == destElem->getSource())
-            {
-                // lock source for read and write
-                pSrcBase = static_cast<unsigned char*>(
-                    srcBuf->lock(HardwareBuffer::HBL_NORMAL));
-                srcInc = srcBuf->getVertexSize();
-                pDestBase = pSrcBase;
-                destInc = srcInc;
-            }
-            else
-            {
-                pSrcBase = static_cast<unsigned char*>(
-                    srcBuf->lock(HardwareBuffer::HBL_READ_ONLY));
-                srcInc = srcBuf->getVertexSize();
-                destBuf = vBind->getBuffer(destElem->getSource());
-                destInc = destBuf->getVertexSize();
-                pDestBase = static_cast<unsigned char*>(
-                    destBuf->lock(HardwareBuffer::HBL_NORMAL));
-            }
-
-		    // find a vertex coord buffer
-		    const VertexElement *elemVPos = vDecl->findElementBySemantic(VES_POSITION);
-            if (elemVPos->getSource() == srcElem->getSource())
-            {
-                pPosBase = pSrcBase;
-                posInc = srcInc;
-            }
-            else if (elemVPos->getSource() == destElem->getSource())
-            {
-                pPosBase = pDestBase;
-                posInc = destInc;
-            }
-            else
-            {
-                // A different buffer
-                posBuf = vBind->getBuffer(elemVPos->getSource());
-                pPosBase = static_cast<unsigned char*>(
-                    posBuf->lock(HardwareBuffer::HBL_READ_ONLY));
-                posInc = posBuf->getVertexSize();
-            }
-			// find a normal buffer
-			const VertexElement *elemVNorm = vDecl->findElementBySemantic(VES_NORMAL);
-			if (!elemVNorm)
-				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
-					"No VES_NORMAL vertex element found", 
-					"Mesh::buildTangentVectors");
-
-			if (elemVNorm->getSource() == srcElem->getSource())
-			{
-				pNormBase = pSrcBase;
-				normInc = srcInc;
-			}
-			else if (elemVNorm->getSource() == destElem->getSource())
-			{
-				pNormBase = pDestBase;
-				normInc = destInc;
-			}
-			else if (elemVNorm->getSource() == elemVPos->getSource())
-			{
-			    // normals are in the same buffer as position
-			    // this condition arises when an animated(skeleton) mesh is not built with 
-			    // an edge list buffer ie no shadows being used.
-			    pNormBase = pPosBase;
-			    normInc = posInc;
-			}
-			else
-			{
-				// A different buffer
-				normBuf = vBind->getBuffer(elemVNorm->getSource());
-				pNormBase = static_cast<unsigned char*>(
-					normBuf->lock(HardwareBuffer::HBL_READ_ONLY));
-				normInc = normBuf->getVertexSize();
-			}
-
-		    size_t numFaces = indexData->indexCount / 3 ;
-
-		    // loop through all faces to calculate the tangents and normals
-		    size_t n;
-		    for (n = 0; n < numFaces; ++n)
-		    {
-			    int i;
-			    for (i = 0; i < 3; ++i)
-			    {
-				    // get indexes of vertices that form a polygon in the position buffer
-				    if (use32bit)
+				// If any vertex splitting happened, we have to give them bone assignments
+				if (getSkeletonName() != StringUtil::BLANK)
+				{
+					for (TangentSpaceCalc::IndexRemapList::iterator r = res.indexesRemapped.begin(); 
+						r != res.indexesRemapped.end(); ++r)
 					{
-						vertInd[i] = *pVIndices32++;
-					}
-					else
-					{
-						vertInd[i] = *pVIndices16++;
-					}
-				    // get the vertices positions from the position buffer
-                    unsigned char* vBase = pPosBase + (posInc * vertInd[i]);
-                    elemVPos->baseVertexPointerToElement(vBase, &pVPos);
-				    vertPos[i].x = pVPos[0];
-				    vertPos[i].y = pVPos[1];
-				    vertPos[i].z = pVPos[2];
-				    // get the vertices tex.coords from the 2D tex.coords buffer
-                    vBase = pSrcBase + (srcInc * vertInd[i]);
-                    srcElem->baseVertexPointerToElement(vBase, &p2DTC);
-				    u[i] = p2DTC[0];
-				    v[i] = p2DTC[1];
-			    }
-			    // calculate the TSB
-                Vector3 tangent = Math::calculateTangentSpaceVector(
-                    vertPos[0], vertPos[1], vertPos[2],
-                    u[0], v[0], u[1], v[1], u[2], v[2]);
-			    // write new tex.coords
-                // note we only write the tangent, not the binormal since we can calculate
-                // the binormal in the vertex program
-			    for (i = 0; i < 3; ++i)
-			    {
-				    // write values (they must be 0 and we must add them so we can average
-                    // all the contributions from all the faces
-                    unsigned char* vBase = pDestBase + (destInc * vertInd[i]);
-                    destElem->baseVertexPointerToElement(vBase, &p3DTC);
-				    p3DTC[0] += tangent.x;
-				    p3DTC[1] += tangent.y;
-				    p3DTC[2] += tangent.z;
-			    }
-		    }
-		    // now loop through all vertices and normalize them
-		    size_t numVerts = usedVertexData->vertexCount ;
-		    for (n = 0; n < numVerts; ++n)
-		    {
-                destElem->baseVertexPointerToElement(pDestBase, &p3DTC);
-			    // read the vertex
-			    Vector3 temp(p3DTC[0], p3DTC[1], p3DTC[2]);
-				// Orthogonalise with the vertex normal since it's currently
-				// orthogonal with the face normals, but will be close to ortho
-				// with vertex normal
-				// Get normal				
-				unsigned char* vBase = pNormBase + (normInc * n);
-				elemVNorm->baseVertexPointerToElement(vBase, &pVNorm);
-				Vector3 normal(pVNorm[0], pVNorm[1], pVNorm[2]);
-				// Apply Gram-Schmidt orthogonalise
-				temp = temp - (normal * normal.dotProduct(temp));
-				
-				// normalize the vertex
-			    temp.normalise();
+						TangentSpaceCalc::IndexRemap& remap = *r;
+						// Copy all bone assignments from the split vertex
+						VertexBoneAssignmentList::const_iterator vbstart = 
+							sm->getBoneAssignments().lower_bound(remap.splitVertex.first);
+						VertexBoneAssignmentList::const_iterator vbend = 
+							sm->getBoneAssignments().upper_bound(remap.splitVertex.first);
+						for (VertexBoneAssignmentList::const_iterator vba = vbstart; vba != vbend; ++vba)
+						{
+							VertexBoneAssignment newAsgn = vba->second;
+							newAsgn.vertexIndex = static_cast<unsigned int>(remap.splitVertex.second);
+							// multimap insert doesn't invalidate iterators
+							sm->addBoneAssignment(newAsgn);
+						}
 
-			    // write it back
-			    p3DTC[0] = temp.x;
-			    p3DTC[1] = temp.y;
-			    p3DTC[2] = temp.z;
+					}
 
-                pDestBase += destInc;
-		    }
-		    // unlock buffers
-            srcBuf->unlock();
-            if (!destBuf.isNull())
-            {
-                destBuf->unlock();
-            }
-            if (!posBuf.isNull())
-            {
-                posBuf->unlock();
-            }
-			if (!normBuf.isNull())
-			{
-				normBuf->unlock();
+				}
 			}
-		    buffIndex->unlock();
-	    }
+		}
 
     }
-
     //---------------------------------------------------------------------
     bool Mesh::suggestTangentVectorBuildParams(VertexElementSemantic targetSemantic,
 		unsigned short& outSourceCoordSet, unsigned short& outIndex)
@@ -1451,7 +1336,8 @@ namespace Ogre {
                         sourceElem = testElem;
                     }
                 }
-                else
+                
+				if(!foundExisting && targetSemantic == VES_TEXTURE_COORDINATES)
                 {
                     // We're looking for the destination
                     // Check to see if we've found a possible
@@ -1567,8 +1453,7 @@ namespace Ogre {
 						s->operationType != RenderOperation::OT_TRIANGLE_LIST && 
 						s->operationType != RenderOperation::OT_TRIANGLE_STRIP)
 					{
-						// Skip this submesh
-						continue;
+                        continue;
 					}
                     if (s->useSharedVertices)
                     {
@@ -1583,7 +1468,7 @@ namespace Ogre {
                                 s->operationType);
                         }
                     }
-                    else
+                    else if(s->isBuildEdgesEnabled())
                     {
                         // own vertex data, add it and reference it directly
                         eb.addVertexData(s->vertexData);
@@ -1618,7 +1503,11 @@ namespace Ogre {
 					LogManager::getSingleton().destroyLog(log);
                 #endif
 				}
-
+				else
+				{
+					// create empty edge data
+					usage.edgeData = OGRE_NEW EdgeData();
+				}
             }
         }
         mEdgeListsBuilt = true;
@@ -1641,7 +1530,7 @@ namespace Ogre {
             {
                 // Only delete if we own this data
                 // Manual LODs > 0 own their own
-                delete usage.edgeData;
+                OGRE_DELETE usage.edgeData;
             }
             usage.edgeData = NULL;
         }
@@ -2053,7 +1942,7 @@ namespace Ogre {
 				"Mesh::createAnimation");
 		}
 
-		Animation* ret = new Animation(name, length);
+		Animation* ret = OGRE_NEW Animation(name, length);
 
 		// Add to list
 		mAnimationsList[name] = ret;
@@ -2125,7 +2014,7 @@ namespace Ogre {
 				"Mesh::getAnimation");
 		}
 
-		delete i->second;
+		OGRE_DELETE i->second;
 
 		mAnimationsList.erase(i);
 
@@ -2137,7 +2026,7 @@ namespace Ogre {
 		AnimationList::iterator i = mAnimationsList.begin();
 		for (; i != mAnimationsList.end(); ++i)
 		{
-			delete i->second;
+			OGRE_DELETE i->second;
 		}
 		mAnimationsList.clear();
 		mAnimationTypesDirty = true;
@@ -2157,7 +2046,7 @@ namespace Ogre {
 	//---------------------------------------------------------------------
 	Pose* Mesh::createPose(ushort target, const String& name)
 	{
-		Pose* retPose = new Pose(target, name);
+		Pose* retPose = OGRE_NEW Pose(target, name);
 		mPoseList.push_back(retPose);
 		return retPose;
 	}
@@ -2200,7 +2089,7 @@ namespace Ogre {
 		}
 		PoseList::iterator i = mPoseList.begin();
 		std::advance(i, index);
-		delete *i;
+		OGRE_DELETE *i;
 		mPoseList.erase(i);
 
 	}
@@ -2211,7 +2100,7 @@ namespace Ogre {
 		{
 			if ((*i)->getName() == name)
 			{
-				delete *i;
+				OGRE_DELETE *i;
 				mPoseList.erase(i);
 				return;
 			}
@@ -2227,7 +2116,7 @@ namespace Ogre {
 	{
 		for (PoseList::iterator i = mPoseList.begin(); i != mPoseList.end(); ++i)
 		{
-			delete *i;
+			OGRE_DELETE *i;
 		}
 		mPoseList.clear();
 	}
