@@ -351,16 +351,6 @@ namespace Ogre {
     {
         mOverlay->_getWorldTransforms(xform);
     }
-    //-----------------------------------------------------------------------
-    const Quaternion& OverlayElement::getWorldOrientation(void) const
-    {
-        return mOverlay->getWorldOrientation();
-    }
-    //-----------------------------------------------------------------------
-    const Vector3& OverlayElement::getWorldPosition(void) const
-    {
-        return mOverlay->getWorldPosition();
-    }
     //---------------------------------------------------------------------
 	void OverlayElement::_positionsOutOfDate(void)
 	{
@@ -566,9 +556,10 @@ namespace Ogre {
         clippingRegion = mClippingRegion;
     }
     //---------------------------------------------------------------------
-    void OverlayElement::_notifyZOrder(ushort newZOrder)
+    ushort OverlayElement::_notifyZOrder(ushort newZOrder)
     {
         mZOrder = newZOrder;
+		return mZOrder + 1;
     }
 
     //---------------------------------------------------------------------
@@ -633,6 +624,12 @@ namespace Ogre {
         }
       
     }
+	//---------------------------------------------------------------------
+	void OverlayElement::visitRenderables(Renderable::Visitor* visitor, 
+		bool debugRenderables)
+	{
+		visitor->visit(this, 0, false);
+	}
     //-----------------------------------------------------------------------
     void OverlayElement::addBaseParameters(void)    
     {
