@@ -57,7 +57,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     ResourceGroupManager::ResourceGroupManager()
-        : mCurrentGroup(0)
+        : mCurrentGroup(0), mLoadingListener(0)
     {
         // Create the 'General' group
         createResourceGroup(DEFAULT_RESOURCE_GROUP_NAME);
@@ -92,8 +92,8 @@ namespace Ogre {
                 "Resource group with name '" + name + "' already exists!", 
                 "ResourceGroupManager::createResourceGroup");
         }
-        ResourceGroup* grp = new ResourceGroup();
-		grp->initialised = false;
+        ResourceGroup* grp = OGRE_NEW_T(ResourceGroup, MEMCATEGORY_RESOURCE)();
+		grp->groupStatus = ResourceGroup::UNINITIALSED;
         grp->name = name;
         grp->worldGeometrySceneManager = 0;
         mResourceGroupMap.insert(
@@ -113,13 +113,15 @@ namespace Ogre {
 		}
 		OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex
 
-		if (!grp->initialised)
+		if (grp->groupStatus == ResourceGroup::UNINITIALSED)
 		{
+			// in the process of initialising
+			grp->groupStatus = ResourceGroup::INITIALISING;
 			// Set current group
 			parseResourceGroupScripts(grp);
 			mCurrentGroup = grp;
 			createDeclaredResources(grp);
-			grp->initialised = true;
+			grp->groupStatus = ResourceGroup::INITIALISED;
 
 			// Reset current group
 			mCurrentGroup = 0;
@@ -137,29 +139,126 @@ namespace Ogre {
 		{
 			ResourceGroup* grp = i->second;
 			OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex
-			if (!grp->initialised)
+			if (grp->groupStatus == ResourceGroup::UNINITIALSED)
 			{
+				// in the process of initialising
+				grp->groupStatus = ResourceGroup::INITIALISING;
 				// Set current group
 				mCurrentGroup = grp;
 				parseResourceGroupScripts(grp);
 				createDeclaredResources(grp);
-				grp->initialised = true;
+				grp->groupStatus = ResourceGroup::INITIALISED;
 				// Reset current group
 				mCurrentGroup = 0;
 			}
 		}
 	}
     //-----------------------------------------------------------------------
+    void ResourceGroupManager::prepareResourceGroup(const String& name, 
+		bool prepareMainResources, bool prepareWorldGeom)
+    {
+		// Can only bulk-load one group at a time (reasonable limitation I think)
+		OGRE_LOCK_AUTO_MUTEX
+
+		LogManager::getSingleton().stream()
+			<< "Preparing resource group '" << name << "' - Resources: "
+			<< prepareMainResources << " World Geometry: " << prepareWorldGeom;
+		// load all created resources
+		ResourceGroup* grp = getResourceGroup(name);
+		if (!grp)
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+				"Cannot find a group named " + name, 
+				"ResourceGroupManager::prepareResourceGroup");
+		}
+
+		OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex 
+		// Set current group
+		mCurrentGroup = grp;
+
+		// Count up resources for starting event
+		ResourceGroup::LoadResourceOrderMap::iterator oi;
+		size_t resourceCount = 0;
+		if (prepareMainResources)
+		{
+			for (oi = grp->loadResourceOrderMap.begin(); oi != grp->loadResourceOrderMap.end(); ++oi)
+			{
+				resourceCount += oi->second->size();
+			}
+		}
+        // Estimate world geometry size
+        if (grp->worldGeometrySceneManager && prepareWorldGeom)
+        {
+            resourceCount += 
+				grp->worldGeometrySceneManager->estimateWorldGeometry(
+					grp->worldGeometry);
+        }
+
+		fireResourceGroupPrepareStarted(name, resourceCount);
+
+		// Now load for real
+		if (prepareMainResources)
+		{
+			for (oi = grp->loadResourceOrderMap.begin(); 
+				oi != grp->loadResourceOrderMap.end(); ++oi)
+			{
+				size_t n = 0;
+				LoadUnloadResourceList::iterator l = oi->second->begin();
+				while (l != oi->second->end())
+				{
+					ResourcePtr res = *l;
+
+					// Fire resource events no matter whether resource needs preparing
+                    // or not. This ensures that the number of callbacks
+					// matches the number originally estimated, which is important
+					// for progress bars.
+					fireResourcePrepareStarted(res);
+
+					// If preparing one of these resources cascade-prepares another resource, 
+					// the list will get longer! But these should be prepared immediately
+					// Call prepare regardless, already prepared or loaded resources will be skipped
+					res->prepare();
+
+					fireResourcePrepareEnded();
+
+					// Did the resource change group? if so, our iterator will have
+					// been invalidated
+					if (res->getGroup() != name)
+					{
+						l = oi->second->begin();
+						std::advance(l, n);
+					}
+					else
+					{
+						++l;
+						++n;
+					}
+				}
+			}
+		}
+        // Load World Geometry
+        if (grp->worldGeometrySceneManager && prepareWorldGeom)
+        {
+            grp->worldGeometrySceneManager->prepareWorldGeometry(
+                grp->worldGeometry);
+        }
+		fireResourceGroupPrepareEnded(name);
+
+		// reset current group
+		mCurrentGroup = 0;
+		
+		LogManager::getSingleton().logMessage("Finished preparing resource group " + name);
+    }
+    //-----------------------------------------------------------------------
     void ResourceGroupManager::loadResourceGroup(const String& name, 
 		bool loadMainResources, bool loadWorldGeom)
     {
 		// Can only bulk-load one group at a time (reasonable limitation I think)
 		OGRE_LOCK_AUTO_MUTEX
 
-		StringUtil::StrStreamType str;
-		str << "Loading resource group '" << name << "' - Resources: "
+		LogManager::getSingleton().stream()
+			<< "Loading resource group '" << name << "' - Resources: "
 			<< loadMainResources << " World Geometry: " << loadWorldGeom;
-		LogManager::getSingleton().logMessage(str.str());
 		// load all created resources
 		ResourceGroup* grp = getResourceGroup(name);
 		if (!grp)
@@ -200,8 +299,8 @@ namespace Ogre {
 				oi != grp->loadResourceOrderMap.end(); ++oi)
 			{
 				size_t n = 0;
-				for (LoadUnloadResourceList::iterator l = oi->second->begin();
-					l != oi->second->end(); ++l, ++n)
+				LoadUnloadResourceList::iterator l = oi->second->begin();
+				while (l != oi->second->end())
 				{
 					ResourcePtr res = *l;
 
@@ -209,13 +308,15 @@ namespace Ogre {
 					// loaded or not. This ensures that the number of callbacks
 					// matches the number originally estimated, which is important
 					// for progress bars.
-					fireResourceStarted(res);
+					fireResourceLoadStarted(res);
 
 					// If loading one of these resources cascade-loads another resource, 
 					// the list will get longer! But these should be loaded immediately
 					// Call load regardless, already loaded resources will be skipped
 					res->load();
 
+					fireResourceLoadEnded();
+
 					// Did the resource change group? if so, our iterator will have
 					// been invalidated
 					if (res->getGroup() != name)
@@ -223,8 +324,11 @@ namespace Ogre {
 						l = oi->second->begin();
 						std::advance(l, n);
 					}
-
-					fireResourceEnded();
+					else
+					{
+						++l;
+						++n;
+					}
 				}
 			}
 		}
@@ -236,6 +340,9 @@ namespace Ogre {
         }
 		fireResourceGroupLoadEnded(name);
 
+		// group is loaded
+		grp->groupStatus = ResourceGroup::LOADED;
+
 		// reset current group
 		mCurrentGroup = 0;
 		
@@ -275,6 +382,8 @@ namespace Ogre {
 			}
 		}
 
+		grp->groupStatus = ResourceGroup::INITIALISED;
+
 		// reset current group
 		mCurrentGroup = 0;
 		LogManager::getSingleton().logMessage("Finished unloading resource group " + name);
@@ -318,6 +427,8 @@ namespace Ogre {
 			}
 		}
 
+		grp->groupStatus = ResourceGroup::INITIALISED;
+
 		// reset current group
 		mCurrentGroup = 0;
 		LogManager::getSingleton().logMessage(
@@ -341,7 +452,7 @@ namespace Ogre {
 		mCurrentGroup = grp;
 		dropGroupContents(grp);
 		// clear initialised flag
-		grp->initialised = false;
+		grp->groupStatus = ResourceGroup::UNINITIALSED;
 		// reset current group
 		mCurrentGroup = 0;
 		LogManager::getSingleton().logMessage("Finished clearing resource group " + name);
@@ -368,6 +479,37 @@ namespace Ogre {
         mResourceGroupMap.erase(mResourceGroupMap.find(name));
 		// reset current group
 		mCurrentGroup = 0;
+    }
+	//-----------------------------------------------------------------------
+	bool ResourceGroupManager::isResourceGroupInitialised(const String& name)
+    {
+		// Can only bulk-destroy one group at a time (reasonable limitation I think)
+		OGRE_LOCK_AUTO_MUTEX
+
+		ResourceGroup* grp = getResourceGroup(name);
+		if (!grp)
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+				"Cannot find a group named " + name, 
+				"ResourceGroupManager::isResourceGroupInitialised");
+		}
+		return (grp->groupStatus != ResourceGroup::UNINITIALSED &&
+			grp->groupStatus != ResourceGroup::INITIALISING);
+    }
+	//-----------------------------------------------------------------------
+	bool ResourceGroupManager::isResourceGroupLoaded(const String& name)
+    {
+		// Can only bulk-destroy one group at a time (reasonable limitation I think)
+		OGRE_LOCK_AUTO_MUTEX
+
+		ResourceGroup* grp = getResourceGroup(name);
+		if (!grp)
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+				"Cannot find a group named " + name, 
+				"ResourceGroupManager::isResourceGroupInitialised");
+		}
+		return (grp->groupStatus == ResourceGroup::LOADED);
     }
     //-----------------------------------------------------------------------
     void ResourceGroupManager::addResourceLocation(const String& name, 
@@ -385,7 +527,7 @@ namespace Ogre {
         // Get archive
         Archive* pArch = ArchiveManager::getSingleton().load( name, locType );
         // Add to location list
-		ResourceLocation* loc = new ResourceLocation();
+		ResourceLocation* loc = OGRE_NEW_T(ResourceLocation, MEMCATEGORY_RESOURCE);
 		loc->archive = pArch;
 		loc->recursive = recursive;
         grp->locationList.push_back(loc);
@@ -463,8 +605,9 @@ namespace Ogre {
                     }
                 }
 				// Erase list entry
-				delete *li;
+				OGRE_DELETE_T(*li, ResourceLocation, MEMCATEGORY_RESOURCE);
 				grp->locationList.erase(li);
+
 				break;
 			}
 
@@ -534,6 +677,14 @@ namespace Ogre {
 		bool searchGroupsIfNotFound, Resource* resourceBeingLoaded)
     {
 		OGRE_LOCK_AUTO_MUTEX
+
+		if(mLoadingListener)
+		{
+			DataStreamPtr stream = mLoadingListener->resourceLoading(resourceName, groupName, resourceBeingLoaded);
+			if(!stream.isNull())
+				return stream;
+		}
+
 		// Try to find in resource index first
 		ResourceGroup* grp = getResourceGroup(groupName);
 		if (!grp)
@@ -552,7 +703,10 @@ namespace Ogre {
 		{
 			// Found in the index
 			pArch = rit->second;
-			return pArch->open(resourceName);
+			DataStreamPtr stream = pArch->open(resourceName);
+			if (mLoadingListener)
+				mLoadingListener->resourceStreamOpened(resourceName, groupName, resourceBeingLoaded, stream);
+			return stream;
 		}
         else 
         {
@@ -564,7 +718,10 @@ namespace Ogre {
             {
                 // Found in the index
                 pArch = rit->second;
-                return pArch->open(resourceName);
+				DataStreamPtr stream = pArch->open(resourceName);
+				if (mLoadingListener)
+					mLoadingListener->resourceStreamOpened(resourceName, groupName, resourceBeingLoaded, stream);
+				return stream;
             }
 		    else
 		    {
@@ -577,6 +734,8 @@ namespace Ogre {
                     if (arch->exists(resourceName))
 				    {
                         DataStreamPtr ptr = arch->open(resourceName);
+						if (mLoadingListener)
+							mLoadingListener->resourceStreamOpened(resourceName, groupName, resourceBeingLoaded, ptr);
 					    return ptr;
 				    }
 			    }
@@ -630,7 +789,8 @@ namespace Ogre {
 
 		// Iterate through all the archives and build up a combined list of
 		// streams
-		DataStreamListPtr ret = DataStreamListPtr(new DataStreamList());
+		// MEMCATEGORY_GENERAL is the only category supported for SharedPtr
+		DataStreamListPtr ret = DataStreamListPtr(OGRE_NEW_T(DataStreamList, MEMCATEGORY_GENERAL)(), SPFM_DELETE_T);
 
 		LocationList::iterator li, liend;
 		liend = grp->locationList.end();
@@ -749,7 +909,8 @@ namespace Ogre {
 			oi != mScriptLoaderOrderMap.end(); ++oi)
 		{
 			ScriptLoader* su = oi->second;
-            FileListListPtr fileListList(new FileListList);
+			// MEMCATEGORY_GENERAL is the only category supported for SharedPtr
+            FileListListPtr fileListList(OGRE_NEW_T(FileListList, MEMCATEGORY_GENERAL)(), SPFM_DELETE_T);
 
 			// Get all the patterns and search them
 			const StringVector& patterns = su->getScriptPatterns();
@@ -777,17 +938,26 @@ namespace Ogre {
 			    // Iterate over each item in the list
 			    for (FileInfoList::iterator fii = (*flli)->begin(); fii != (*flli)->end(); ++fii)
 			    {
-                    LogManager::getSingleton().logMessage(
-                        "Parsing script " + fii->filename);
-                    fireScriptStarted(fii->filename);
-                    {
+					bool skipScript = false;
+                    fireScriptStarted(fii->filename, skipScript);
+					if(skipScript)
+					{
+						LogManager::getSingleton().logMessage(
+							"Skipping script " + fii->filename);
+					}
+					else
+					{
+						LogManager::getSingleton().logMessage(
+							"Parsing script " + fii->filename);
                         DataStreamPtr stream = fii->archive->open(fii->filename);
                         if (!stream.isNull())
                         {
+							if (mLoadingListener)
+								mLoadingListener->resourceStreamOpened(fii->filename, grp->name, 0, stream);
                             su->parseScript(stream, grp->name);
                         }
                     }
-				    fireScriptEnded(fii->filename);
+					fireScriptEnded(fii->filename, skipScript);
 			    }
             }
 		}
@@ -815,7 +985,7 @@ namespace Ogre {
 			LoadUnloadResourceList* loadList;
 			if (li == grp->loadResourceOrderMap.end())
 			{
-				loadList = new LoadUnloadResourceList();
+				loadList = OGRE_NEW_T(LoadUnloadResourceList, MEMCATEGORY_RESOURCE)();
 				grp->loadResourceOrderMap[mgr->getLoadingOrder()] = loadList;
 			}
 			else
@@ -953,7 +1123,7 @@ namespace Ogre {
 		LoadUnloadResourceList* loadList;
 		if (i == grp.loadResourceOrderMap.end())
 		{
-			loadList = new LoadUnloadResourceList();
+			loadList = OGRE_NEW_T(LoadUnloadResourceList, MEMCATEGORY_RESOURCE)();
 			grp.loadResourceOrderMap[order] = loadList;
 		}
 		else
@@ -1013,7 +1183,7 @@ namespace Ogre {
 			{
 				(*k)->getCreator()->remove((*k)->getHandle());
 			}
-			delete j->second;
+			OGRE_DELETE_T(j->second, LoadUnloadResourceList, MEMCATEGORY_RESOURCE);
 		}
         grp->loadResourceOrderMap.clear();
 
@@ -1034,18 +1204,18 @@ namespace Ogre {
 		    {
 			    // Don't iterate over resources to drop with ResourceManager
 			    // Assume this is being done anyway since this is a shutdown method
-			    delete j->second;
+			    OGRE_DELETE_T(j->second, LoadUnloadResourceList, MEMCATEGORY_RESOURCE);
 		    }
 		    // Drop location list
 		    for (LocationList::iterator ll = grp->locationList.begin();
 			    ll != grp->locationList.end(); ++ll)
 		    {
-			    delete *ll;
+			    OGRE_DELETE_T(*ll, ResourceLocation, MEMCATEGORY_RESOURCE);
 		    }
         }
 
 		// delete ResourceGroup
-		delete grp;
+		OGRE_DELETE_T(grp, ResourceGroup, MEMCATEGORY_RESOURCE);
 	}
 	//-----------------------------------------------------------------------
 	void ResourceGroupManager::fireResourceGroupScriptingStarted(const String& groupName, size_t scriptCount)
@@ -1058,23 +1228,26 @@ namespace Ogre {
 		}
 	}
 	//-----------------------------------------------------------------------
-	void ResourceGroupManager::fireScriptStarted(const String& scriptName)
+	void ResourceGroupManager::fireScriptStarted(const String& scriptName, bool &skipScript)
 	{
 		OGRE_LOCK_AUTO_MUTEX
 		for (ResourceGroupListenerList::iterator l = mResourceGroupListenerList.begin();
 			l != mResourceGroupListenerList.end(); ++l)
 		{
-			(*l)->scriptParseStarted(scriptName);
+			bool temp = false;
+			(*l)->scriptParseStarted(scriptName, temp);
+			if(temp)
+				skipScript = true;
 		}
 	}
     //-----------------------------------------------------------------------
-    void ResourceGroupManager::fireScriptEnded(const String& scriptName)
+    void ResourceGroupManager::fireScriptEnded(const String& scriptName, bool skipped)
     {
         OGRE_LOCK_AUTO_MUTEX
             for (ResourceGroupListenerList::iterator l = mResourceGroupListenerList.begin();
                 l != mResourceGroupListenerList.end(); ++l)
             {
-                (*l)->scriptParseEnded(scriptName);
+                (*l)->scriptParseEnded(scriptName, skipped);
             }
     }
 	//-----------------------------------------------------------------------
@@ -1098,7 +1271,7 @@ namespace Ogre {
 		}
 	}
 	//-----------------------------------------------------------------------
-	void ResourceGroupManager::fireResourceStarted(const ResourcePtr& resource)
+	void ResourceGroupManager::fireResourceLoadStarted(const ResourcePtr& resource)
 	{
 		OGRE_LOCK_AUTO_MUTEX
 		for (ResourceGroupListenerList::iterator l = mResourceGroupListenerList.begin();
@@ -1108,7 +1281,7 @@ namespace Ogre {
 		}
 	}
     //-----------------------------------------------------------------------
-    void ResourceGroupManager::fireResourceEnded(void)
+    void ResourceGroupManager::fireResourceLoadEnded(void)
     {
         OGRE_LOCK_AUTO_MUTEX
             for (ResourceGroupListenerList::iterator l = mResourceGroupListenerList.begin();
@@ -1148,6 +1321,46 @@ namespace Ogre {
 		}
 	}
 	//-----------------------------------------------------------------------
+	void ResourceGroupManager::fireResourceGroupPrepareStarted(const String& groupName, size_t resourceCount)
+	{
+		OGRE_LOCK_AUTO_MUTEX
+		for (ResourceGroupListenerList::iterator l = mResourceGroupListenerList.begin();
+			l != mResourceGroupListenerList.end(); ++l)
+		{
+			(*l)->resourceGroupPrepareStarted(groupName, resourceCount);
+		}
+	}
+    //-----------------------------------------------------------------------
+    void ResourceGroupManager::fireResourcePrepareStarted(const ResourcePtr& resource)
+    {
+        OGRE_LOCK_AUTO_MUTEX
+        for (ResourceGroupListenerList::iterator l = mResourceGroupListenerList.begin();
+            l != mResourceGroupListenerList.end(); ++l)
+        {
+            (*l)->resourcePrepareStarted(resource);
+        }
+    }
+    //-----------------------------------------------------------------------
+    void ResourceGroupManager::fireResourcePrepareEnded(void)
+    {
+        OGRE_LOCK_AUTO_MUTEX
+            for (ResourceGroupListenerList::iterator l = mResourceGroupListenerList.begin();
+                l != mResourceGroupListenerList.end(); ++l)
+            {
+                (*l)->resourcePrepareEnded();
+            }
+    }
+	//-----------------------------------------------------------------------
+	void ResourceGroupManager::fireResourceGroupPrepareEnded(const String& groupName)
+	{
+		OGRE_LOCK_AUTO_MUTEX
+		for (ResourceGroupListenerList::iterator l = mResourceGroupListenerList.begin();
+			l != mResourceGroupListenerList.end(); ++l)
+		{
+			(*l)->resourceGroupPrepareEnded(groupName);
+		}
+	}
+	//-----------------------------------------------------------------------
     void ResourceGroupManager::shutdownAll(void)
     {
         OGRE_LOCK_AUTO_MUTEX
@@ -1163,7 +1376,8 @@ namespace Ogre {
     StringVectorPtr ResourceGroupManager::listResourceNames(const String& groupName, bool dirs)
     {
         OGRE_LOCK_AUTO_MUTEX
-        StringVectorPtr vec(new StringVector());
+		// MEMCATEGORY_GENERAL is the only category supported for SharedPtr
+        StringVectorPtr vec(OGRE_NEW_T(StringVector, MEMCATEGORY_GENERAL)(), SPFM_DELETE_T);
 
         // Try to find in resource index first
         ResourceGroup* grp = getResourceGroup(groupName);
@@ -1193,7 +1407,8 @@ namespace Ogre {
     FileInfoListPtr ResourceGroupManager::listResourceFileInfo(const String& groupName, bool dirs)
     {
         OGRE_LOCK_AUTO_MUTEX
-        FileInfoListPtr vec(new FileInfoList());
+		// MEMCATEGORY_GENERAL is the only category supported for SharedPtr
+        FileInfoListPtr vec(OGRE_NEW_T(FileInfoList, MEMCATEGORY_GENERAL)(), SPFM_DELETE_T);
 
         // Try to find in resource index first
         ResourceGroup* grp = getResourceGroup(groupName);
@@ -1223,7 +1438,8 @@ namespace Ogre {
         const String& pattern, bool dirs)
     {
         OGRE_LOCK_AUTO_MUTEX
-        StringVectorPtr vec(new StringVector());
+		// MEMCATEGORY_GENERAL is the only category supported for SharedPtr
+		StringVectorPtr vec(OGRE_NEW_T(StringVector, MEMCATEGORY_GENERAL)(), SPFM_DELETE_T);
 
         // Try to find in resource index first
         ResourceGroup* grp = getResourceGroup(groupName);
@@ -1252,7 +1468,8 @@ namespace Ogre {
         const String& pattern, bool dirs)
     {
         OGRE_LOCK_AUTO_MUTEX
-        FileInfoListPtr vec(new FileInfoList());
+		// MEMCATEGORY_GENERAL is the only category supported for SharedPtr
+        FileInfoListPtr vec(OGRE_NEW_T(FileInfoList, MEMCATEGORY_GENERAL)(), SPFM_DELETE_T);
 
         // Try to find in resource index first
         ResourceGroup* grp = getResourceGroup(groupName);
@@ -1335,7 +1552,65 @@ namespace Ogre {
 		return false;
 
 	}
-    //-----------------------------------------------------------------------
+	//-----------------------------------------------------------------------
+	time_t ResourceGroupManager::resourceModifiedTime(const String& groupName, const String& resourceName)
+	{
+		OGRE_LOCK_AUTO_MUTEX
+
+		// Try to find in resource index first
+		ResourceGroup* grp = getResourceGroup(groupName);
+		if (!grp)
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+				"Cannot locate a resource group called '" + groupName + "'", 
+				"ResourceGroupManager::resourceModifiedTime");
+		}
+
+		return resourceModifiedTime(grp, resourceName);
+
+	}
+	//-----------------------------------------------------------------------
+	time_t ResourceGroupManager::resourceModifiedTime(ResourceGroup* grp, const String& resourceName)
+	{
+		OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex
+
+		// Try indexes first
+		ResourceLocationIndex::iterator rit = grp->resourceIndexCaseSensitive.find(resourceName);
+		if (rit != grp->resourceIndexCaseSensitive.end())
+		{
+			return rit->second->getModifiedTime(resourceName);
+		}
+        else 
+        {
+            // try case insensitive
+            String lcResourceName = resourceName;
+            StringUtil::toLowerCase(lcResourceName);
+            rit = grp->resourceIndexCaseInsensitive.find(lcResourceName);
+            if (rit != grp->resourceIndexCaseInsensitive.end())
+            {
+				return rit->second->getModifiedTime(resourceName);
+            }
+		    else
+		    {
+			    // Search the hard way
+			    LocationList::iterator li, liend;
+			    liend = grp->locationList.end();
+			    for (li = grp->locationList.begin(); li != liend; ++li)
+			    {
+				    Archive* arch = (*li)->archive;
+					time_t testTime = arch->getModifiedTime(resourceName);
+
+                    if (testTime > 0)
+				    {
+						return testTime;
+				    }
+			    }
+		    }
+        }
+
+		return 0;
+	}
+	//-----------------------------------------------------------------------
 	ResourceGroupManager::ResourceGroup* 
 	ResourceGroupManager::findGroupContainingResourceImpl(const String& filename)
 	{
@@ -1431,6 +1706,16 @@ namespace Ogre {
 		OGRE_LOCK_MUTEX(grp->OGRE_AUTO_MUTEX_NAME) // lock group mutex
 		return grp->resourceDeclarations;
 	}
+	//-------------------------------------------------------------------------
+	void ResourceGroupManager::setLoadingListener(ResourceLoadingListener *listener)
+	{
+		mLoadingListener = listener;
+	}
+	//-------------------------------------------------------------------------
+	ResourceLoadingListener *ResourceGroupManager::getLoadingListener()
+	{
+		return mLoadingListener;
+	}
 	//-----------------------------------------------------------------------
 	ScriptLoader::~ScriptLoader()
 	{
