@@ -658,8 +658,9 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Root::removeFrameListener(FrameListener* oldListener)
     {
-        // Remove, 1 only (set)
-        mRemovedFrameListeners.insert(oldListener);
+		// Remove, 1 only (set), and only when this listener was added before.
+		if( mFrameListeners.find( oldListener ) != mFrameListeners.end() )
+			mRemovedFrameListeners.insert(oldListener);
     }
     //-----------------------------------------------------------------------
     bool Root::_fireFrameStarted(FrameEvent& evt)
@@ -999,8 +1000,8 @@ namespace Ogre {
         if (!mActiveRenderer)
         {
             OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
-            "Cannot create window - no render "
-            "system has been selected.", "Root::destroyRenderWindow");
+            "Cannot detach target - no render "
+            "system has been selected.", "Root::detachRenderTarget");
         }
 
         mActiveRenderer->detachRenderTarget( target->getName() );
@@ -1011,8 +1012,8 @@ namespace Ogre {
         if (!mActiveRenderer)
         {
             OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
-            "Cannot create window - no render "
-            "system has been selected.", "Root::destroyRenderWindow");
+            "Cannot detach target - no render "
+            "system has been selected.", "Root::detachRenderTarget");
         }
 
         mActiveRenderer->detachRenderTarget( name );
@@ -1023,8 +1024,8 @@ namespace Ogre {
         if (!mActiveRenderer)
         {
             OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
-            "Cannot create window - no render "
-            "system has been selected.", "Root::getRenderWindow");
+            "Cannot get target - no render "
+            "system has been selected.", "Root::getRenderTarget");
         }
 
         return mActiveRenderer->getRenderTarget(name);
@@ -1067,17 +1068,21 @@ namespace Ogre {
 		// Load plugin library
         DynLib* lib = DynLibManager::getSingleton().load( pluginName );
 		// Store for later unload
-		mPluginLibs.push_back(lib);
+		// Check for existence, because if called 2+ times DynLibManager returns existing entry
+		if (std::find(mPluginLibs.begin(), mPluginLibs.end(), lib) == mPluginLibs.end())
+		{
+			mPluginLibs.push_back(lib);
 
-		// Call startup function
-		DLL_START_PLUGIN pFunc = (DLL_START_PLUGIN)lib->getSymbol("dllStartPlugin");
+			// Call startup function
+			DLL_START_PLUGIN pFunc = (DLL_START_PLUGIN)lib->getSymbol("dllStartPlugin");
 
-		if (!pFunc)
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Cannot find symbol dllStartPlugin in library " + pluginName,
-				"Root::loadPlugin");
+			if (!pFunc)
+				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Cannot find symbol dllStartPlugin in library " + pluginName,
+					"Root::loadPlugin");
 
-		// This must call installPlugin
-		pFunc();
+			// This must call installPlugin
+			pFunc();
+		}
 
 	}
     //-----------------------------------------------------------------------
